@@ -1,3 +1,5 @@
+# encoding=utf-8
+
 import blocks
 
 
@@ -29,35 +31,51 @@ def _init_states():
             return None
 
     number = State('digit')
+    fullwidth_num = State('digit')
     basic_latin = State('en')
+    fullwidth_latin = State('en')
     ext_latin = State('latin')
     basic_cjk = State('cjk')
     ja = State('ja')
     kr = State('kr')
 
     number['DIGIT'] = lambda _: number
+    fullwidth_num['FULLWIDTH_DIGIT'] = lambda _: fullwidth_num
+
     basic_latin['BASIC_LATIN'] = lambda _: basic_latin
     basic_latin['BASIC_PUNCTUATION'] = (
         lambda c: basic_latin if c == "'" or c == '-' else None)
     basic_latin['LATIN_EXTENDED_LETTER'] = lambda _: ext_latin
+
+    fullwidth_latin['FULLWIDTH_LATIN'] = lambda _: fullwidth_latin
+    fullwidth_latin['HALFWIDTH_AND_FULLWIDTH_FORMS'] = (
+        lambda c: fullwidth_latin if c == u'＇' or c == u'－' else None)
+    fullwidth_latin['BASIC_PUNCTUATION'] = (
+        lambda c: fullwidth_latin if c == "'" or c == '-' else None)
+
     ext_latin['BASIC_LATIN'] = lambda _: ext_latin
     ext_latin['BASIC_PUNCTUATION'] = (
         lambda c: ext_latin if c == "'" or c == '-' else None)
     ext_latin['LATIN_EXTENDED_LETTER'] = lambda _: ext_latin
+
     basic_cjk['CJK_UNIFIED_IDEOGRAPHS'] = lambda _: basic_cjk
     basic_cjk['HIRAGANA'] = lambda _: ja
     basic_cjk['KATAKANA'] = lambda _: ja
+
     ja['CJK_UNIFIED_IDEOGRAPHS'] = lambda _: ja
     ja['HIRAGANA'] = lambda _: ja
     ja['KATAKANA'] = lambda _: ja
+
     kr['HANGUL_SYLLABLES'] = lambda _: kr
     kr['HANGUL_JAMO_EXTENDED_B'] = lambda _: kr
 
     return State(None, {
         'DIGIT': lambda _: number,
+        'FULLWIDTH_DIGIT': lambda _: fullwidth_num,
+        'BASIC_LATIN': lambda _: basic_latin,
+        'FULLWIDTH_LATIN': lambda _: fullwidth_latin,
         'HANGUL_SYLLABLES': lambda _: kr,
         'HANGUL_JAMO_EXTENDED_B': lambda _: kr,
-        'BASIC_LATIN': lambda _: basic_latin,
         'CJK_UNIFIED_IDEOGRAPHS': lambda _: basic_cjk,
         'HIRAGANA': lambda _: ja,
         'KATAKANA': lambda _: ja,
